"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var forms_1 = require("@angular/forms");
var validation_methods_1 = require("./validation-methods");
/*
 The Vadacl class can be used as the superclass for a component that implements validation, or as an injected service.
 */
var Vadacl = (function () {
    function Vadacl() {
    }
    Vadacl.prototype.applyRules = function (domainClass, propertyName, validationOverrides) {
        var validators = [];
        //Collect the property validations defined in the domain class (if any).
        var propertyValidations = (domainClass && domainClass.validations) ? (domainClass.validations[propertyName] || {}) : {};
        //Apply any validation overrides and additional validations.
        var mergedValidations = this.mergeValidations(propertyValidations, validationOverrides);
        for (var mv in mergedValidations) {
            //Add className and propertyName values to arguments
            mergedValidations[mv].className = domainClass ? domainClass.constructor.name : undefined;
            mergedValidations[mv].propertyName = propertyName ? propertyName : undefined;
            //Check that validation method exists
            if (Vadacl.validationMethods[mv]) {
                //Parse the argument values to apply to the validation method.
                var validatorArguments = this.getValidatorArguments(this.getMethodDeclaredArguments(Vadacl.validationMethods[mv]), mergedValidations[mv]);
                /*
                 Execute the validation method, which will return a validator that the Angular reactive form classes will
                 trigger on a value change, and add that validator to the validators array.
                 */
                validators.push(Vadacl.validationMethods[mv].apply(null, validatorArguments));
            }
            else {
                throw "*** Error thrown by Vadacl.applyRules: Validation method \"" + mv + "\" is undefined in ValidationMethods. ***";
            }
        }
        return validators;
    };
    Vadacl.prototype.applyCollectionRule = function (domainClass, propertyName, validationOverride) {
        var collectionValidator;
        var propertyValidation = (domainClass && domainClass.validations) ? (domainClass.validations[propertyName] || {}) : {};
        var mergedValidation = this.mergeValidations(propertyValidation, validationOverride);
        var methodList = Object.keys(mergedValidation);
        if (methodList.length > 1) {
            throw "*** Error thrown by Vadacl.applyCollectionRule: A single validation method must be applied. ***";
        }
        else if (methodList.length === 1) {
            var mv = methodList[0];
            //Add className and propertyName values to arguments
            mergedValidation[mv].className = domainClass ? domainClass.constructor.name : undefined;
            mergedValidation[mv].propertyName = propertyName ? propertyName : undefined;
            //Check that validation method exists
            if (Vadacl.validationMethods[mv]) {
                //Parse the argument values to apply to the validation method.
                var validatorArguments = this.getValidatorArguments(this.getMethodDeclaredArguments(Vadacl.validationMethods[mv]), mergedValidation[mv]);
                /*
                 Execute the validation method, which will return a validator that the Angular reactive form classes will
                 trigger on a value change.
                 */
                collectionValidator = Vadacl.validationMethods[mv].apply(null, validatorArguments);
            }
            else {
                throw "*** Error thrown by Vadacl.applyCollectionRule: Validation method \"" + mv + "\" is undefined in ValidationMethods. ***";
            }
        }
        return collectionValidator;
    };
    /*
     Converts function to a string and then uses regular expressions to pull out the argument names.  The argument names
     are denoted as object literal properties with values equal to their position in the list of arguments (the positions
     are needed to help place the validation method argument values in the correct order).
     */
    Vadacl.prototype.getMethodDeclaredArguments = function (fn) {
        var methodDeclarationRegExp = /^function\s*[^\(]*\(\s*([^\)]*)\)/m;
        var argumentSplit = /,/;
        var stripCommentsRegExp = /((\/\/.*$)|(\/\*[\s\S]*?\*\/))/mg;
        var methodArguments = {};
        var argumentPosition = 0;
        var methodAsString = fn.toString().replace(stripCommentsRegExp, '');
        var methodDeclaration = methodAsString.match(methodDeclarationRegExp);
        var argumentArray = methodDeclaration[1].split(argumentSplit);
        if (argumentArray[0] != '') {
            for (var argIndex in argumentArray) {
                methodArguments[argumentArray[argIndex].replace(/\s/g, '')] = argumentPosition++;
            }
        }
        return methodArguments;
    };
    /*
     The arguments passed into the call to the validation method are organized in the proper order based on the order
     of the argument names in the validation method.
     */
    Vadacl.prototype.getValidatorArguments = function (argumentDeclarations, validatorArguments) {
        var executionArguments = [];
        for (var arg in argumentDeclarations) {
            executionArguments[argumentDeclarations[arg]] = validatorArguments[arg] || null;
        }
        return executionArguments;
    };
    /*
     Merges the previous validation settings with any new settings.
     */
    Vadacl.prototype.mergeValidations = function (baseValidations, overrideValidations) {
        for (var validation in overrideValidations) {
            if (baseValidations[validation] == undefined) {
                baseValidations[validation] = overrideValidations[validation];
            }
            else {
                for (var setting in overrideValidations[validation]) {
                    baseValidations[validation][setting] = overrideValidations[validation][setting];
                }
            }
        }
        return baseValidations;
    };
    /*
     Determines whether the errors for the form element should be displayed.  By default, returns false
     if the element is still marked as untouched.
     */
    Vadacl.prototype.showErrors = function (formElement, onlyAfterTouched) {
        if (onlyAfterTouched === void 0) { onlyAfterTouched = true; }
        var elementActive = onlyAfterTouched ? formElement.touched : true;
        return (formElement.dirty && formElement.invalid && elementActive) ? true : false;
    };
    /*
     Returns array of error messages. By default, error messages only returned when control is dirty.
     */
    Vadacl.prototype.getControlErrors = function (control, onlyWhenDirty) {
        if (onlyWhenDirty === void 0) { onlyWhenDirty = true; }
        var errorMessages = [];
        if ((!onlyWhenDirty || control.dirty) && control.errors) {
            var errorArray = Object.keys(control.errors);
            for (var err in errorArray) {
                if (control.errors[errorArray[err]].message) {
                    errorMessages.push(control.errors[errorArray[err]].message);
                }
            }
        }
        return errorMessages;
    };
    /*
     Ensures a programmatic change to an AbstractControl value is marked as dirty (and by default as touched)
     prior to the change, properly invoking validation and the display of any validation errors.
     */
    Vadacl.prototype.changeControlValue = function (control, value, markTouched) {
        if (markTouched === void 0) { markTouched = true; }
        control.markAsDirty();
        if (markTouched) {
            control.markAsTouched();
        }
        ;
        control.setValue(value);
    };
    /*
     Convenience method that returns a FormGroup based on the domain class argument with the desired FormControls and validations
     */
    Vadacl.prototype.generateForm = function (domainClass, mods) {
        if (mods === void 0) { mods = {}; }
        var generatorMap = {};
        var formMap = {};
        //If mods argument includes an array of the "only" properties you want in the form, use that. Otherwise, grab the domain class properties
        var domainProperties = mods.only ? mods.only : Object.keys(domainClass);
        for (var _i = 0, domainProperties_1 = domainProperties; _i < domainProperties_1.length; _i++) {
            var p = domainProperties_1[_i];
            if (p != 'validations') {
                generatorMap[p] = {
                    currentValue: domainClass[p],
                    //Use key/values pairs in mods.rename to name the FormControl something different from the property name
                    controlName: mods.rename && mods.rename[p] ? mods.rename[p] : p,
                    //Apply any additional validations for the property in mods.validations, otherwise the normal property validations will be used
                    validations: mods.validations && mods.validations[p] ? this.applyRules(domainClass, p, mods.validations[p]) : this.applyRules(domainClass, p),
                    //If mods.only is not defined, check mods.exclude for exclusions from the form.
                    include: (!mods.only && mods.exclude && mods.exclude.indexOf(p) > -1) ? false : true
                };
            }
        }
        for (var f in generatorMap) {
            if (generatorMap[f].include) {
                formMap[generatorMap[f].controlName] = new forms_1.FormControl(generatorMap[f].currentValue, generatorMap[f].validations);
            }
        }
        return new forms_1.FormGroup(formMap);
    };
    Vadacl.validationMethods = validation_methods_1.ValidationMethods;
    return Vadacl;
}());
exports.Vadacl = Vadacl;
//# sourceMappingURL=vadacl.js.map