"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var forms_1 = require("@angular/forms");
var validation_messages_1 = require("./validation-messages");
/*
 The ValidationMethods class contains all of the validation methods defined for use by vadacl.  Validation methods
 provided in Angular are wrapped and leveraged whenever practical.
 */
var ValidationMethods = (function () {
    function ValidationMethods() {
    }
    ValidationMethods.getLocaleMessage = function (validationErrorName, className, propertyName) {
        if (className && propertyName && this.messages[className] && this.messages[className][propertyName]) {
            return this.messages[className][propertyName][validationErrorName] || this.messages[validationErrorName];
        }
        else {
            return this.messages[validationErrorName];
        }
    };
    //Modeled after the official Angular Required validators value checks
    ValidationMethods.isEmpty = function (control) {
        var v = control.value;
        if (v === undefined || v === null || (typeof v === 'string' && v == '')) {
            return true;
        }
        else {
            return false;
        }
    };
    ValidationMethods.required = function (message, className, propertyName) {
        var msg = message || ValidationMethods.getLocaleMessage('required', className, propertyName);
        return function (control) {
            return ValidationMethods.isEmpty(control) ? { 'required': { 'isEmpty': true, 'message': msg } } : null;
        };
    };
    ValidationMethods.requiredTrue = function (message, className, propertyName) {
        var msg = message || ValidationMethods.getLocaleMessage('requiredtrue', className, propertyName);
        return function (control) {
            return control.value === true ? null : { 'requiredtrue': { 'isNotTrue': true, 'message': msg } };
        };
    };
    ValidationMethods.minLength = function (minLength, message, className, propertyName) {
        var msg = message || ValidationMethods.getLocaleMessage('minlength', className, propertyName);
        return function (control) {
            //Use and invoke the official Angular minLength Validator
            var baseValidator = forms_1.Validators.minLength(minLength);
            var outcome = baseValidator(control);
            if (outcome) {
                //Append the message
                outcome['minlength'].message = msg;
            }
            return outcome;
        };
    };
    ValidationMethods.maxLength = function (maxLength, message, className, propertyName) {
        var msg = message || ValidationMethods.getLocaleMessage('maxlength', className, propertyName);
        return function (control) {
            //Use and invoke the official Angular maxLength Validator
            var baseValidator = forms_1.Validators.maxLength(maxLength);
            var outcome = baseValidator(control);
            if (outcome) {
                //Append the message
                outcome['maxlength'].message = msg;
            }
            return outcome;
        };
    };
    ValidationMethods.min = function (min, message, className, propertyName) {
        var msg = message || ValidationMethods.getLocaleMessage('min', className, propertyName);
        return function (control) {
            //Use and invoke the official Angular min Validator, added in 4.x
            var baseValidator = forms_1.Validators.min(min);
            var outcome = baseValidator(control);
            if (outcome) {
                //Append the message
                outcome['min'].message = msg;
            }
            return outcome;
        };
    };
    ValidationMethods.max = function (max, message, className, propertyName) {
        var msg = message || ValidationMethods.getLocaleMessage('max', className, propertyName);
        return function (control) {
            //Use and invoke the official Angular max Validator, added in 4.x
            var baseValidator = forms_1.Validators.max(max);
            var outcome = baseValidator(control);
            if (outcome) {
                //Append the message
                outcome['max'].message = msg;
            }
            return outcome;
        };
    };
    ValidationMethods.email = function (message, className, propertyName) {
        var msg = message || ValidationMethods.getLocaleMessage('email', className, propertyName);
        return function (control) {
            //When a value is present, use and invoke the official Angular email Validator, added in 4.x
            return (!ValidationMethods.isEmpty(control) && forms_1.Validators.email(control)) ? { 'email': { 'isNotEmail': true, 'message': msg } } : null;
        };
    };
    ValidationMethods.pattern = function (pattern, message, className, propertyName) {
        var msg = message || ValidationMethods.getLocaleMessage('pattern', className, propertyName);
        return function (control) {
            //Use and invoke the official Angular pattern Validator
            var baseValidator = forms_1.Validators.pattern(pattern);
            var outcome = baseValidator(control);
            if (outcome) {
                //Append the message
                outcome['pattern'].message = msg;
            }
            return outcome;
        };
    };
    /*
     Validates that the length of the value of the AbstractControl falls within a certain range.  Can be used to validate
     the length of a string or the number of form controls in a FormGroup or FormArray.
     */
    ValidationMethods.withinLength = function (minLength, maxLength, message, className, propertyName) {
        var msg = message || ValidationMethods.getLocaleMessage('withinlength', className, propertyName);
        return function (control) {
            //Do not display if the control is empty
            if (ValidationMethods.isEmpty(control)) {
                return null;
            }
            var v = control.value;
            return v.length < minLength || v.length > maxLength ?
                { 'withinlength': { 'minLength': minLength, 'maxLength': maxLength, 'actualLength': v.length, 'message': msg } } :
                null;
        };
    };
    /*
     Validates that the sum of the numeric values of the FormGroup or FormArray controls equals a certain amount.  Should
     be applied with the applyCollectionRule() method of the Vadacl class.
     */
    ValidationMethods.totals = function (total, message, className, propertyName) {
        var msg = message || ValidationMethods.getLocaleMessage('totals', className, propertyName);
        return function (controlCollection) {
            var valueTotal;
            if (controlCollection.controls instanceof Array) {
                for (var c in controlCollection.controls) {
                    if (valueTotal === undefined && controlCollection.controls[c].value) {
                        valueTotal = +controlCollection.controls[c].value;
                    }
                    else if (controlCollection.controls[c].value) {
                        valueTotal += +controlCollection.controls[c].value;
                    }
                }
            }
            else {
                var controlNames = Object.keys(controlCollection.controls);
                for (var cn in controlNames) {
                    var currentControl = controlCollection.controls[controlNames[cn]];
                    if (valueTotal === undefined && currentControl.value) {
                        valueTotal = +currentControl.value;
                    }
                    else if (currentControl.value) {
                        valueTotal += +currentControl.value;
                    }
                }
            }
            return total === valueTotal ? null :
                { 'totals': { 'requiredTotal': total, 'actualTotal': valueTotal, 'message': msg } };
        };
    };
    /*
     Validates that all of the values of the FormControls within a FormGroup or FormArray are exactly equal.  Should
     be applied with the applyCollectionRule() method of the Vadacl class.
     */
    ValidationMethods.equalValues = function (message, className, propertyName) {
        var msg = message || ValidationMethods.getLocaleMessage('equalvalues', className, propertyName);
        return function (controlCollection) {
            var areEqual = true;
            var firstValue;
            if (controlCollection.controls instanceof Array) {
                for (var c in controlCollection.controls) {
                    if (firstValue === undefined) {
                        firstValue = controlCollection.controls[c].value;
                    }
                    else {
                        if (firstValue !== controlCollection.controls[c].value) {
                            areEqual = false;
                        }
                    }
                }
            }
            else {
                var controlNames = Object.keys(controlCollection.controls);
                for (var cn in controlNames) {
                    var currentControl = controlCollection.controls[controlNames[cn]];
                    if (firstValue === undefined) {
                        firstValue = currentControl.value;
                    }
                    else {
                        if (firstValue !== currentControl.value) {
                            areEqual = false;
                        }
                    }
                }
            }
            return (firstValue && areEqual === true) ? null : { 'equalvalues': { 'message': msg } };
        };
    };
    /*
     Validates that the number of FormControls within a FormGroup or FormArray with a value of Boolean true falls within
     a given range.  Designed primarily to validate how many checkboxes are checked.  Should be applied with the
     applyCollectionRule() method of the Vadacl class.
     */
    ValidationMethods.withinTrueCount = function (minCount, maxCount, message, className, propertyName) {
        var msg = message || ValidationMethods.getLocaleMessage('withintruecount', className, propertyName);
        return function (controlCollection) {
            var trueCount = 0;
            var minimumTrue = minCount ? minCount : 0;
            var withinCount = false;
            if (controlCollection.controls instanceof Array) {
                for (var c in controlCollection.controls) {
                    if (controlCollection.controls[c].value === true) {
                        trueCount++;
                    }
                }
            }
            else {
                var controlNames = Object.keys(controlCollection.controls);
                for (var cn in controlNames) {
                    var currentControl = controlCollection.controls[controlNames[cn]];
                    if (currentControl.value === true) {
                        trueCount++;
                    }
                }
            }
            if (maxCount) {
                withinCount = (trueCount >= minimumTrue && trueCount <= maxCount) ? true : false;
            }
            else {
                withinCount = trueCount >= minimumTrue ? true : false;
            }
            return withinCount ? null : { 'withintruecount': { 'minTrue': minimumTrue, 'maxTrue': maxCount, 'trueCount': trueCount, 'message': msg } };
        };
    };
    ValidationMethods.messages = validation_messages_1.ValidationMessages;
    return ValidationMethods;
}());
exports.ValidationMethods = ValidationMethods;
//# sourceMappingURL=validation-methods.js.map